package game.block;

import java.util.List;

import game.audio.SoundType;
import game.collect.Lists;
import game.color.Colorizer;
import game.entity.npc.EntityNPC;
import game.init.Blocks;
import game.init.Config;
import game.init.ItemRegistry;
import game.init.WoodType;
import game.item.CheatTab;
import game.item.Item;
import game.item.ItemShears;
import game.item.ItemStack;
import game.material.Material;
import game.properties.IProperty;
import game.properties.PropertyBool;
import game.properties.PropertyEnum;
import game.renderer.blockmodel.ModelBlock;
import game.renderer.particle.ParticleType;
import game.rng.Random;
import game.tileentity.TileEntity;
import game.world.BlockPos;
import game.world.IWorldAccess;
import game.world.State;
import game.world.World;
import game.world.WorldClient;
import game.world.WorldServer;

public class BlockLeaves extends BlockLeavesBase
{
    public static final PropertyEnum<LeavesType> TYPE = PropertyEnum.<LeavesType>create("type", LeavesType.class);
    public static final PropertyBool DECAY = PropertyBool.create("decay");
    public static final List<BlockLeaves> LEAVES = Lists.newArrayList();

	private final WoodType type;
	
    int[] surroundings;

    public BlockLeaves(WoodType type)
    {
        super(Material.leaves);
		this.type = type;
		this.setDefaultState(this.getBaseState().withProperty(TYPE, LeavesType.SPRING).withProperty(DECAY, Boolean.valueOf(true)));
        this.setTickRandomly();
        this.setTab(CheatTab.tabPlants);
        this.setHardness(0.2F);
        this.setLightOpacity(1);
        this.setStepSound(SoundType.GRASS);
        LEAVES.add(this);
    }

//    public int getBlockColor()
//    {
//        return ColorizerFoliage.getFoliageColor(0.5D, 1.0D);
//    }

//    public int getRenderColor(IBlockState state)
//    {
//        return ColorizerFoliage.getFoliageColorBasic();
//    }
//
//    public int colorMultiplier(IWorldAccess worldIn, BlockPos pos, int renderPass)
//    {
//        return BiomeColorHelper.getFoliageColorAtPos(worldIn, pos);
//    }

    public void onBlockRemoved(WorldServer worldIn, BlockPos pos, State state)
    {
        int i = 1;
        int j = i + 1;
        int k = pos.getX();
        int l = pos.getY();
        int i1 = pos.getZ();

        if (worldIn.isAreaLoaded(new BlockPos(k - j, l - j, i1 - j), new BlockPos(k + j, l + j, i1 + j)))
        {
            for (int j1 = -i; j1 <= i; ++j1)
            {
                for (int k1 = -i; k1 <= i; ++k1)
                {
                    for (int l1 = -i; l1 <= i; ++l1)
                    {
                        BlockPos blockpos = pos.add(j1, k1, l1);
                        State iblockstate = worldIn.getState(blockpos);

                        if (iblockstate.getBlock().getMaterial() == Material.leaves && !((Boolean)iblockstate.getValue(DECAY)).booleanValue())
                        {
                            worldIn.setState(blockpos, iblockstate.withProperty(DECAY, Boolean.valueOf(true)), 4);
                        }
                    }
                }
            }
        }
    }

    public void updateTick(WorldServer worldIn, BlockPos pos, State state, Random rand)
    {
		if(Config.seasonLeaves && state.getValue(TYPE) != worldIn.getLeavesGen(pos)) {
			worldIn.setState(pos, state.withProperty(TYPE, worldIn.getLeavesGen(pos)), 2);
		}
    	if(Config.leafDry && worldIn.getTemperatureC(pos) >= 50.0f) {
    		worldIn.setState(pos, worldIn.rand.chance(40) ? Blocks.air.getState() : Blocks.dry_leaves.getState());
    	}
//        if (!worldIn.client)
//        {
    	else if (Config.leavesDecay && ((Boolean)state.getValue(DECAY)).booleanValue())
        {
            int i = 4;
            int j = i + 1;
            int k = pos.getX();
            int l = pos.getY();
            int i1 = pos.getZ();
            int j1 = 32;
            int k1 = j1 * j1;
            int l1 = j1 / 2;

            if (this.surroundings == null)
            {
                this.surroundings = new int[j1 * j1 * j1];
            }

            if (worldIn.isAreaLoaded(new BlockPos(k - j, l - j, i1 - j), new BlockPos(k + j, l + j, i1 + j)))
            {
                BlockPos.MutableBlockPos blockpos$mutableblockpos = new BlockPos.MutableBlockPos();

                for (int i2 = -i; i2 <= i; ++i2)
                {
                    for (int j2 = -i; j2 <= i; ++j2)
                    {
                        for (int k2 = -i; k2 <= i; ++k2)
                        {
                            Block block = worldIn.getState(blockpos$mutableblockpos.set(k + i2, l + j2, i1 + k2)).getBlock();

                            if (!(block instanceof BlockLog))
                            {
                                if (block.getMaterial() == Material.leaves)
                                {
                                    this.surroundings[(i2 + l1) * k1 + (j2 + l1) * j1 + k2 + l1] = -2;
                                }
                                else
                                {
                                    this.surroundings[(i2 + l1) * k1 + (j2 + l1) * j1 + k2 + l1] = -1;
                                }
                            }
                            else
                            {
                                this.surroundings[(i2 + l1) * k1 + (j2 + l1) * j1 + k2 + l1] = 0;
                            }
                        }
                    }
                }

                for (int i3 = 1; i3 <= 4; ++i3)
                {
                    for (int j3 = -i; j3 <= i; ++j3)
                    {
                        for (int k3 = -i; k3 <= i; ++k3)
                        {
                            for (int l3 = -i; l3 <= i; ++l3)
                            {
                                if (this.surroundings[(j3 + l1) * k1 + (k3 + l1) * j1 + l3 + l1] == i3 - 1)
                                {
                                    if (this.surroundings[(j3 + l1 - 1) * k1 + (k3 + l1) * j1 + l3 + l1] == -2)
                                    {
                                        this.surroundings[(j3 + l1 - 1) * k1 + (k3 + l1) * j1 + l3 + l1] = i3;
                                    }

                                    if (this.surroundings[(j3 + l1 + 1) * k1 + (k3 + l1) * j1 + l3 + l1] == -2)
                                    {
                                        this.surroundings[(j3 + l1 + 1) * k1 + (k3 + l1) * j1 + l3 + l1] = i3;
                                    }

                                    if (this.surroundings[(j3 + l1) * k1 + (k3 + l1 - 1) * j1 + l3 + l1] == -2)
                                    {
                                        this.surroundings[(j3 + l1) * k1 + (k3 + l1 - 1) * j1 + l3 + l1] = i3;
                                    }

                                    if (this.surroundings[(j3 + l1) * k1 + (k3 + l1 + 1) * j1 + l3 + l1] == -2)
                                    {
                                        this.surroundings[(j3 + l1) * k1 + (k3 + l1 + 1) * j1 + l3 + l1] = i3;
                                    }

                                    if (this.surroundings[(j3 + l1) * k1 + (k3 + l1) * j1 + (l3 + l1 - 1)] == -2)
                                    {
                                        this.surroundings[(j3 + l1) * k1 + (k3 + l1) * j1 + (l3 + l1 - 1)] = i3;
                                    }

                                    if (this.surroundings[(j3 + l1) * k1 + (k3 + l1) * j1 + l3 + l1 + 1] == -2)
                                    {
                                        this.surroundings[(j3 + l1) * k1 + (k3 + l1) * j1 + l3 + l1 + 1] = i3;
                                    }
                                }
                            }
                        }
                    }
                }
            }

            int l2 = this.surroundings[l1 * k1 + l1 * j1 + l1];

            if (l2 >= 0)
            {
                worldIn.setState(pos, state.withProperty(DECAY, Boolean.valueOf(false)), 4);
            }
            else
            {
                this.destroy(worldIn, pos);
            }
        }
//        }
    }

    public void randomDisplayTick(WorldClient worldIn, BlockPos pos, State state, Random rand)
    {
        if (worldIn.isRainingAt(pos.up(), true) && !worldIn.isBlockSolid(pos.down()) && rand.chance(15)) // == 1
        {
            double d0 = (double)((float)pos.getX() + rand.floatv());
            double d1 = (double)pos.getY() - 0.05D;
            double d2 = (double)((float)pos.getZ() + rand.floatv());
            worldIn.spawnParticle(ParticleType.DRIP_WATER, d0, d1, d2, 0.0D, 0.0D, 0.0D);
        }
    }

    private void destroy(World worldIn, BlockPos pos)
    {
        this.dropBlockAsItem(worldIn, pos, worldIn.getState(pos), 0);
        worldIn.setBlockToAir(pos);
    }

    /**
     * Returns the quantity of items to drop on block destruction.
     */
    public int quantityDropped(Random random)
    {
        return random.chance(0, 1, this.type.getSaplingChance());
    }

    /**
     * Get the Item that this Block should drop when harvested.
     */
    public Item getItemDropped(State state, Random rand, int fortune)
    {
        return ItemRegistry.getRegisteredItem(this.type.getName() + "_sapling");
    }

    /**
     * Spawns this Block's drops into the World as EntityItems.
     */
    public void dropBlockAsItemWithChance(World worldIn, BlockPos pos, State state, float chance, int fortune)
    {
        if (!worldIn.client)
        {
            int i = this.type.getSaplingChance();

            if (fortune > 0)
            {
                i -= 2 << fortune;

                if (i < 10)
                {
                    i = 10;
                }
            }

            if (worldIn.rand.chance(i))
            {
                Item item = this.getItemDropped(state, worldIn.rand, fortune);
                spawnAsEntity(worldIn, pos, new ItemStack(item, 1, this.damageDropped(state)));
            }

            i = 200;

            if (fortune > 0)
            {
                i -= 10 << fortune;

                if (i < 40)
                {
                    i = 40;
                }
            }

            if(this.type.getItem() != null && worldIn.rand.chance(i)) // np
				spawnAsEntity(worldIn, pos, new ItemStack(ItemRegistry.getRegisteredItem(this.type.getItem()), 1, 0));
        }
    }

    public int getRenderColor(State state)
    {
        return state.getBlock() != this || !state.getValue(TYPE).isTinted() ? 16777215 : 
        		((this.type.getTintType() == null ? Colorizer.BASIC : this.type.getTintType()).getColor());
    }

    public int colorMultiplier(IWorldAccess worldIn, BlockPos pos, int renderPass)
    {
    	State state = worldIn.getState(pos);
        return state.getBlock() != this || !state.getValue(TYPE).isTinted() ? 16777215 : (this.type.getTintType()
        		== null ? Colorizer.getFoliageColor(worldIn, pos) : this.type.getTintType().getColor());
    }

	public void getSubBlocks(Item itemIn, CheatTab tab, List<ItemStack> list) {
		for(int z = 0; z < LeavesType.values().length; z++) {
			list.add(new ItemStack(itemIn, 1, z));
		}
	}

	public ItemStack createStackedBlock(State state) {
		return new ItemStack(ItemRegistry.getItemFromBlock(this), 1, ((LeavesType)state.getValue(TYPE)).getIndex());
	}

	public State getStateFromMeta(int meta) {
		return this.getState().withProperty(TYPE, LeavesType.getById(meta & 7)).withProperty(DECAY, Boolean.valueOf((meta & 8) == 0));
	}

	public int getMetaFromState(State state) {
		int i = 0;
		i = i | ((LeavesType)state.getValue(TYPE)).getIndex();

		if(!((Boolean)state.getValue(DECAY)).booleanValue()) {
			i |= 8;
		}

		return i;
	}

//	public BlockPlanks.EnumType getWoodType() {
//		return this.type;
//	}

	protected IProperty[] getProperties() {
		return new IProperty[] {TYPE, DECAY};
	}

//	public int damageDropped(IBlockState state) {
//		return ((LeavesType)state.getValue(TYPE)).getIndex();
//	}

	public void harvestBlock(World worldIn, EntityNPC player, BlockPos pos, State state, TileEntity te) {
		if(!worldIn.client && player.getCurrentEquippedItem() != null && player.getCurrentEquippedItem().getItem() instanceof ItemShears) {
//			player.triggerAchievement(StatRegistry.mineBlockStatArray[BlockRegistry.getIdFromBlock(this)]);
			spawnAsEntity(worldIn, pos, new ItemStack(ItemRegistry.getItemFromBlock(this), 1, ((LeavesType)state.getValue(TYPE)).getIndex()));
		}
		else {
			super.harvestBlock(worldIn, player, pos, state, te);
		}
	}

    public ModelBlock getModel(String name, State state) {
    	return state.getValue(TYPE).isTinted() ? new ModelBlock(name + "_" + state.getValue(TYPE).getName()).add().all().tint() :
    			new ModelBlock(name + "_" + state.getValue(TYPE).getName()).add().all();
    }

    public IProperty<?>[] getIgnoredProperties() {
    	return new IProperty[] {DECAY};
    }
    
    public int getDamageValue(World worldIn, BlockPos pos) {
    	State state = worldIn.getState(pos);
        return state.getBlock() == this ? state.getValue(TYPE).getIndex() : super.getDamageValue(worldIn, pos);
    }
}
