package game.command;

import java.util.List;
import java.util.Map;

import game.collect.Lists;
import game.collect.Maps;

public class CachedExecutable {
	private final Executable executable;
	private final String name;
	private final Map<String, Parameter> parameters;
	private final List<Parameter> positionals;
	
	protected CachedExecutable(Executable executable, Map<String, Parameter> parameters, List<Parameter> positionals) {
		this.executable = executable;
		this.parameters = parameters;
		this.positionals = positionals;
		this.name = executable.getName();
	}
	
	public static CachedExecutable cacheExecutable(Executable executable) {
		Map<String, Parameter> parameters = Maps.newTreeMap();
		Map<String, Parameter> params = executable.getParameters();
		parameters.putAll(params);
		Parameter[] positions = new Parameter[parameters.size()];
		int pos = -1;
		for(Parameter param : params.values()) {
			if(param.isPositional()) {
				positions[param.getPosition()] = param;
				pos = param.getPosition() > pos ? param.getPosition() : pos;
			}
			if(param.hasShortName())
				parameters.put("" + param.getShortName(), param);
		}
		List<Parameter> positionals = Lists.newArrayList();
		for(int z = 0; z <= pos; z++) {
			if(positions[z] == null)
				throw new NullPointerException("positions[" + z + "]");
			positionals.add(positions[z]);
		}
		return new CachedExecutable(executable, parameters, positionals);
	}
	
	public Executable getExecutable() {
		return this.executable;
	}
	
	public Map<String, Parameter> getParameters() {
		return this.parameters;
	}
	
	public List<Parameter> getPositionals() {
		return this.positionals;
	}
	
	public String getName() {
		return this.name;
	}
}
