package game.gui;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileFilter;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.Arrays;
import java.util.List;
import java.util.Set;

import javax.imageio.ImageIO;

import game.Game;
import game.audio.SoundManager;
import game.collect.Lists;
import game.collect.Sets;
import game.entity.npc.Alignment;
import game.entity.npc.CharacterInfo;
import game.entity.npc.EntityNPC;
import game.entity.npc.ModelPart;
import game.entity.npc.SpeciesInfo;
import game.gui.element.Button;
import game.gui.element.Slider;
import game.gui.element.TextField;
import game.gui.element.Slider.SliderReceiver;
import game.gui.inventory.GuiInventory;
import game.init.EntityRegistry;
import game.init.SpeciesRegistry;
import game.init.SpeciesRegistry.ModelType;
import game.io.FileUtils;
import game.io.Log;
import game.lib.input.Keyboard;
import game.network.NetHandlerPlayServer;
import game.packet.CPacketAction;
import game.packet.CPacketMessage;
import game.renderer.FontRenderer;
import game.renderer.GlState;
import game.renderer.texture.DynamicTexture;
import game.renderer.texture.EntityTexManager;
import game.renderer.texture.TextureUtil;
import game.util.Predicate;

public class GuiSkin extends GuiScreen
{
	private static class SkinEntry implements GuiListExtended.IGuiListEntry
	{
	    private static final String SKINS_BUTTONS = "textures/gui/server_selection.png";
	    
	    private final GuiSkin owner;
	    private final Game gm;
	    private final File skinFile;
	    private final String id;
	    private final ModelType model;
	    private final CharacterInfo charinfo;
	    private final BufferedImage skinImage;
	    private final String icon;
	    private DynamicTexture dyntex;
	    private long clickTime;

	    protected SkinEntry(GuiSkin parent, String id, File file, CharacterInfo charinfo, BufferedImage image, ModelType model)
	    {
	        this.owner = parent;
	        this.skinFile = file;
	    	this.charinfo = charinfo;
	    	this.id = id;
	        this.model = model;
	        this.gm = Game.getGame();
	        if(this.skinFile != null) // {
	            this.skinImage = image;
//	            this.skin = skin.getName().substring(0, skin.getName().length() - 4);
//	        }
	        else // {
	            this.skinImage = null;
//	        	this.skin = null;
//	        }
	        if(this.skinImage != null) {
	        	this.icon = "skins/" + this.id + "/icon";
		        this.dyntex = new DynamicTexture(this.skinImage);
		        this.gm.getTextureManager().loadTexture(this.icon, this.dyntex);
	        }
	        else {
	        	this.icon = null;
		        this.dyntex = null;
	        }
	    }

	    public void drawEntry(int slotIndex, int x, int y, int listWidth, int slotHeight, int mouseX, int mouseY, boolean isSelected)
	    {
//	        FontRenderer.drawString(this.skin == null ? "---" : this.skin, x + 32 + 3, y + 1, 16777215);
	        List<String> list = FontRenderer.listFormattedStringToWidth((this.skinFile != null ? this.skinFile.getName() 
	        		: (this.charinfo == null ? "Standard" : (
	        				(this.charinfo.species.prefix && this.charinfo.spclass != null && this.charinfo.spclass.type != null ? 
	        				this.charinfo.spclass.type.toString() : 
	        				EntityRegistry.getEntityName(this.charinfo.species.id))
	        				+ (this.charinfo.name.isEmpty() ? "" : (" " + this.charinfo.name)) + " (" + this.charinfo.skin + ")"))) 
//	        		+ TextColor.LIGHT_GRAY + " (" + TextColor.GREEN +
//	        		this.model.display + TextColor.LIGHT_GRAY +")"
	        		, listWidth - 16 - 2);

	        for (int i = 0; i < Math.min(list.size(), 2); ++i)
	        {
	            FontRenderer.drawString((String)list.get(i), x + 16 + 3, y + FontRenderer.FONT_HEIGHT * i, this.charinfo == null ? 
	            		(this.skinFile != null ? 0xffffff : 0xc0c0c0) : this.charinfo.color1 | this.charinfo.color2);
	        }

	        GlState.color(1.0F, 1.0F, 1.0F, 1.0F);

//	        if(this.model != ModelType.HUMANOID) {
//	        	this.drawTextureAt(x, y, EntitySkinManager.HEAD_DEF);
//	        }
	        if (this.dyntex != null)
	        {
	            this.drawTextureAt(x, y, this.icon);
	        }
	        else if(this.charinfo == null)
	        {
	            this.drawTextureAt(x, y, EntityTexManager.getDefault(this.model));
	        }
	        else
	        {
	            this.drawTextureAt(x, y, EntityNPC.getSkinTexture(this.charinfo.skin.startsWith("~") ? 
	            		this.charinfo.skin.substring(1) : this.charinfo.skin));
	        }
	        
	        if (/* this.gm.touchscreen || */ isSelected)
	        {
	            this.gm.getTextureManager().bindTexture(SKINS_BUTTONS);
	            Gui.drawRect(x, y, x + 16, y + 16, -1601138544);
	            GlState.color(1.0F, 1.0F, 1.0F, 1.0F);
	            int k1 = mouseX - x;
	            int l1 = mouseY - y;

	            if (k1 < 16 && k1 > 0)
	            {
	                Gui.drawScaledCustomSizeModalRect(x, y, 0.0F, 32.0F, 32, 32, 16, 16, 256.0F, 256.0F);
	            }
	        }
	    }

	    protected void drawTextureAt(int x, int y, String tex)
	    {
	        this.gm.getTextureManager().bindTexture(tex);
	        GlState.enableBlend();
	        this.gm.getRenderManager().getRenderObject(this.model).renderHead(x, y, this.model.texWidth, this.model.texHeight, 2);
	        GlState.disableBlend();
	    }
	    
	    public void deleteTexture() {
	    	if(this.dyntex != null) {
	    		this.gm.getTextureManager().deleteTexture(this.icon);
	    		this.dyntex = null;
	    	}
	    }

	    /**
	     * Returns true if the mouse has been pressed on this control.
	     */
	    public boolean mousePressed(int slotIndex, int p_148278_2_, int p_148278_3_, int p_148278_4_, int p_148278_5_, int p_148278_6_)
	    {
	    	if (Game.getSystemTime() - this.clickTime < 250L)
	        {
	        	this.owner.close();
	    		return true;
	        }
	        else {
	            BufferedImage img = this.skinImage;
	            boolean slim = false;
	            if(this.charinfo != null) {
	            	try {
	            		img = TextureUtil.readImage(FileUtils.getStream(
	            				EntityNPC.getSkinTexture(this.charinfo.skin.startsWith("~") ? this.charinfo.skin.substring(1) : 
	            				this.charinfo.skin)));
	            	}
	            	catch(IOException e) {
	            		if(e instanceof FileNotFoundException)
							Log.warn("Textur für Skin ist nicht vorhanden: " + 
							EntityNPC.getSkinTexture(this.charinfo.skin.startsWith("~") ? this.charinfo.skin.substring(1) : 
							this.charinfo.skin));
						else
	                		Log.error("Konnte Textur nicht laden", e);
	            		return false;
	            	}
	            	slim = this.charinfo.skin.startsWith("~");
	            }
	            else if(this.skinFile != null) {
	            	slim = this.skinFile.getName().startsWith("slim_");
	            }
	            this.owner.selectEntry(slotIndex, slim);
	    		this.owner.selectSkin(this.id, img, slim, this.model);
	    		if(p_148278_5_ < 16 && p_148278_5_ > 0)
	    			this.owner.close();
	            this.clickTime = Game.getSystemTime();
	            return false;
	        }
	    }

	    public File getFile()
	    {
	        return this.skinFile;
	    }

	    public boolean canConvert()
	    {
	        return this.skinFile != null && this.model == ModelType.HUMANOID;
	    }

	    public String getLocation()
	    {
	        return this.charinfo == null ? null : (this.charinfo.skin.startsWith("~") ? this.charinfo.skin.substring(1) : this.charinfo.skin);
	    }

	    public boolean canCopy()
	    {
	        return this.charinfo != null;
	    }
	}

	private static class SkinSelector extends GuiListExtended
	{
	    private final GuiSkin owner;
	    private final List<SkinEntry> skinList = Lists.<SkinEntry>newArrayList();
	    private int selectedSlotIndex = -1;

	    public SkinSelector(GuiSkin ownerIn, ModelType model, String currentSkin, Game gmIn, int widthIn, int heightIn, int topIn, int bottomIn, int slotHeightIn)
	    {
	        super(gmIn, widthIn, heightIn, topIn, bottomIn, slotHeightIn);
	        this.owner = ownerIn;
//	        this.setBackgroundDrawing(false);
	        this.load(currentSkin, model);
	    }
	    
	    public static BufferedImage loadSkin(File file)
	    {
	    	BufferedImage img = null;
	        try {
	        	img = ImageIO.read(file);
	        }
	        catch(IOException e) {
	        	return null;
	        }
	        if(img == null) {
	        	return null;
	        }
//	        if(img.getWidth() != 64 || img.getHeight() != 64) { // || img.getType() != BufferedImage.TYPE_INT_ARGB) {
//	        	return null;
//	        }
	        return img;
	    }

	    public void load(String currentSkin, ModelType model)
	    {
	        this.unload();
	        File[] files = new File("skins").listFiles(new FileFilter() {
				public boolean accept(File pathname) {
					return pathname.isFile() && pathname.getName().endsWith(".png");
				}
			});
	        int pos = 0;
//	        for(ModelType model : ModelType.values()) {
        	this.skinList.add(new SkinEntry(this.owner, "default", null, null, null, model));
    		if("default".equals(currentSkin))
    			this.setSelectedSlotIndex(pos);
    		pos++;
//	        }
	        if(files != null) {
		        Arrays.sort(files);
		        for(File file : files)
		        {
		        	BufferedImage img = loadSkin(file);
		        	if(img != null) {
//		        		for(ModelType model : ModelType.values()) {
	        			if(img.getWidth() == model.texWidth && img.getHeight() == model.texHeight) {
			        		this.skinList.add(new SkinEntry(this.owner, file.getName(), file, null, img, model));
			        		if(file.getName().equals(currentSkin))
			        			this.setSelectedSlotIndex(pos);
			        		pos++;
	        			}
//		        		}
		        	}
		        }
	        }
	        for(SpeciesInfo species : SpeciesRegistry.SPECIMEN) {
		        for(CharacterInfo charinfo : species.chars) {
		        	if(charinfo.species.renderer == model) {
		//        for(Entry<String, ModelType> entry : SpeciesRegistry.SKINS.entrySet()) {
			        	this.skinList.add(new SkinEntry(this.owner, charinfo.skin, null, charinfo, null, charinfo.species.renderer));
			    		if(charinfo.skin.equals(currentSkin))
			    			this.setSelectedSlotIndex(pos);
			    		pos++;
		        	}
		        }
	        }
	    }

	    public void unload()
	    {
	        for (SkinEntry entry : this.skinList)
	        {
	            entry.deleteTexture();
	        }
	        
	        this.skinList.clear();
	    }

	    protected int getScrollBarX()
	    {
	        return 0;
	    }

	    /**
	     * Gets the width of the list
	     */
	    public int getListWidth()
	    {
	        return 274 - 20;
	    }

	    protected boolean isInList(int x, int y)
	    {
	    	return x >= 6;
	    }
	}
	
	private class DragAdjust extends Button {
		private int mouseX;
		private int mouseY;
	    private float yawOffset;
	    private float pitchOffset;
		public boolean dragging;

		public DragAdjust(int id, int x, int y, int w, int h) {
			super(id, x, y, w, h, "");
		}

		public void drawButton(Game gm, int mouseX, int mouseY) {
			if(this.visible && this.dragging) {
				GuiSkin.this.yaw = this.yawOffset + (this.mouseX - mouseX) * 2.0f;
				GuiSkin.this.pitch = this.pitchOffset + (this.mouseY - mouseY) * 2.0f;
			}
			if(this.visible)
		        drawRect(this.xPosition, this.yPosition, this.xPosition + this.width, this.yPosition + this.height, 0x20ffffff);
		}

		public boolean mousePressed(Game gm, int mouseX, int mouseY) {
			if(super.mousePressed(gm, mouseX, mouseY)) {
				this.mouseX = mouseX;
				this.mouseY = mouseY;
				this.yawOffset = GuiSkin.this.yaw;
				this.pitchOffset = GuiSkin.this.pitch;
				this.dragging = true;
				return true;
			}
			else {
				return false;
			}
		}

		public void mouseReleased(int mouseX, int mouseY) {
			this.dragging = false;
		}

		public void playPressSound(SoundManager soundHandlerIn) {
		}
	}

	private class ButtonPart extends Button
    {
        private final ModelPart part;

        private ButtonPart(int id, int x, int y, int width, int height, ModelPart part)
        {
            super(id, x, y, width, height, GuiSkin.this.formatPart(part));
            this.part = part;
        }
    }

	private class ButtonAlign extends Button
    {
        private final Alignment align;

        private ButtonAlign(int id, int x, int y, int width, int height, Alignment align)
        {
            super(id, x, y, width, height, align.color + align.display);
            this.align = align;
        }
    }
	
	private class HeightReceiver implements SliderReceiver {
		private int height;
		
		private HeightReceiver(int value) {
			this.height = value;
		}
		
		public float getValue(Game gm) {
			return this.height;
		}
		
		public void setValue(Game gm, float value) {
			this.height = (int)value;
	        if(gm.thePlayer != null)
	            gm.thePlayer.sendQueue.addToSendQueue(new CPacketAction(CPacketAction.Action.SET_HEIGHT, this.height));
		}
		
		public String formatValue(Game gm) {
			return "Spieler-Größe: " + (int)this.height + " cm";
		}
	}

    private final Set<ModelPart> parts = Sets.newHashSet(ModelPart.values());
    private final GuiScreen parentScreen;
    
    private SkinSelector listSelector;
    private Button convertButton;
    private Button templateButton;
    private Button modelButton;
    private DragAdjust adjust;
    private TextField nameField;
    private boolean initialized;
//    private int update;
    private float yaw;
    private float pitch;

    public GuiSkin(GuiScreen parentScreen)
    {
        this.parentScreen = parentScreen;
    }
    
    public void updateScreen()
    {
//    	if(this.update > 0)
//    		this.update -= 1;
    	this.nameField.updateCursorCounter();
//        if(this.gm.nickChanged) {
//        	this.nameField.setText(this.gm.nickname);
//        	this.gm.nickChanged = false;
//        }
    }
    
    public void initGui()
    {
        Keyboard.enableRepeatEvents(true);
        this.buttonList.clear();

        if (!this.initialized)
        {
            this.initialized = true;
            this.listSelector = new SkinSelector(this, this.gm.thePlayer == null ? ModelType.HUMANOID : this.gm.thePlayer.getModel(),  this.gm.skinId.isEmpty() ? null : this.gm.skinId, 
            		this.gm, 274, this.height, 32, this.height - 32, 20);
        }
        else
        {
            this.listSelector.setDimensions(274, this.height, 32, this.height - 32);
        }

        this.buttonList.add(new Button(8, 164, this.height - 28, 274 - 164, 20, "Neu laden"));
        int flags = this.gm.thePlayer == null ? ~ModelPart.ARMS_SLIM.getMask() : this.gm.thePlayer.getModelParts();
        this.parts.clear();
        for(ModelPart part : ModelPart.values()) {
            if((flags & part.getMask()) == part.getMask())
            	this.parts.add(part);
        }
        this.buttonList.add(this.convertButton = new Button(9, 10, this.height - 28, 150, 20, 
        		"Konvertieren: " + 
        		(this.parts.contains(ModelPart.ARMS_SLIM) ? "Schlank" : "Standard")));
        this.buttonList.add(this.templateButton = new Button(10, this.width - 274, this.height - 28, 150, 20, 
        		"Vorlage kopieren"));
        this.buttonList.add(this.adjust = new DragAdjust(20, 274 + 10, 64, this.width - 274 - 274 - 20, this.height - 128));
        for (int z = 0; z < ModelPart.values().length; z++)
        {
        	ModelPart part = ModelPart.values()[z];
        	Button button = new ButtonPart(z + 500, this.width - 274 + (z & 1) * 134, 32 + 24 * (z >> 1), 130, 20, part);
        	if(part == ModelPart.ARMS_SLIM)
        		this.modelButton = button;
	        this.buttonList.add(button);
        }
        for (int z = 0; z < Alignment.values().length; z++)
        {
        	Button button;
        	this.buttonList.add(button = new ButtonAlign(z + 600, this.width - 274 + (z % 3) * 89, this.height - 158 + 23 * (z / 3), 86, 20, 
        			Alignment.values()[z]));
        	button.enabled = this.gm.thePlayer == null || this.gm.thePlayer.getAlignment() != Alignment.values()[z];
        }
//        this.gm.setFloatValue(Options.PLAYER_HEIGHT, );
        this.buttonList.add(new Slider(30, this.width - 274, this.height - 28 - 54, 264, 20, 
        		new HeightReceiver(this.gm.thePlayer == null ? 180 : (int)(this.gm.thePlayer.getSpecies().size * this.gm.thePlayer.getHeight() * 100.0f + 0.5f)),
        		120.0f, 320.0f, 1.0f)
//        	protected void mouseDragged(Game gm, int mouseX, int mouseY) {
//        		if(this.visible && this.dragging)
//        			GuiSkin.this.update = 2;
//        		super.mouseDragged(gm, mouseX, mouseY);
//        	}
//
//        	public boolean mouseScrolled(Game gm, int mouseX, int mouseY, int offset) {
//        		if(super.mouseScrolled(gm, mouseX, mouseY, offset)) {
//        			GuiSkin.this.update = 2;
//        			return true;
//        		}
//        		return false;
//        	}
        );
        this.buttonList.add(new Button(0, this.width - 274 + 154, this.height - 28, 264 - 154, 20, "Fertig"));
        this.convertButton.enabled = false;
        this.templateButton.enabled = false;
    	this.nameField = new TextField(this.width - 274 + 2, this.height + 2 - 28 - 24, 260, 16);
        this.nameField.setMaxStringLength(32);
        this.nameField.setText(this.gm.thePlayer == null ? "" : this.gm.thePlayer.getCustomNameTag());
        this.nameField.setValidator(new Predicate<String>()
        {
            public boolean apply(String name)
            {
                return NetHandlerPlayServer.isValidNick(name);
            }
        });
//    	this.gm.nickChanged = false;
    }
    
    private void sendNameToServer()
    {
    	String name = this.nameField.getText();
    	if(name.isEmpty())
    		this.nameField.setText(this.gm.thePlayer == null ? "..." : this.gm.thePlayer.getCustomNameTag());
    	else if(this.gm.thePlayer != null)
            this.gm.thePlayer.sendQueue.addToSendQueue(new CPacketMessage(CPacketMessage.Type.DISPLAY, name));
    }
    
    public void selectSkin(String id, BufferedImage img, boolean slim, ModelType model)
    {
    	this.gm.setSkin(img, id, model, slim);
    }
    
    public void close() {
    	this.gm.displayGuiScreen(this.parentScreen);
    }

    public void selectEntry(int index, boolean slim)
    {
        this.listSelector.setSelectedSlotIndex(index);
    	if(this.listSelector.getSelectedIndex() >= 0) {
            SkinEntry entry = this.listSelector.getListEntry(this.listSelector.getSelectedIndex());
            if(slim != this.parts.contains(ModelPart.ARMS_SLIM))
				try {
					this.actionPerformed(this.modelButton);
				}
				catch(IOException e) {
				}
//        	this.update = 2;
            this.convertButton.enabled = entry.canConvert();
            this.templateButton.enabled = entry.canCopy();
    	}
    	else {
            this.convertButton.enabled = false;
            this.templateButton.enabled = false;
    	}
    }
    
    public boolean doesGuiPauseGame()
    {
        return false;
    }
    
    public void handleMouseInput() throws IOException
    {
        super.handleMouseInput();
        if(!this.adjust.dragging)
        	this.listSelector.handleMouseInput();
    }
    
    public void onGuiClosed()
    {
        Keyboard.enableRepeatEvents(false);

        this.listSelector.unload();
    }
    
    protected void actionPerformed(Button button) throws IOException
    {
        if (button.enabled)
        {
            if (button.id == 0)
            {
                this.gm.displayGuiScreen(this.parentScreen);
            }
            else if (button.id == 8)
            {
                this.refreshList();
            }
            else if (button.id == 9)
            {
            	if(this.listSelector.getSelectedIndex() >= 0) {
	            	SkinEntry entry = this.listSelector.getListEntry(this.listSelector.getSelectedIndex());
	            	if(entry.getFile() != null && SkinConverter.convertSkin(entry.getFile(), new File("skins"), 
	            			this.parts.contains(ModelPart.ARMS_SLIM))) {
                    	this.refreshList();
                    }
            	}
            }
            else if (button.id == 10)
            {
            	if(this.listSelector.getSelectedIndex() >= 0) {
	            	SkinEntry entry = this.listSelector.getListEntry(this.listSelector.getSelectedIndex());
	            	if(entry.getLocation() != null) {
	            		String loc = (this.parts.contains(ModelPart.ARMS_SLIM) ? "slim_" : "") + entry.getLocation();
	            		File file = new File(new File("skins"), loc + ".png");
	            		int z = 1;
	            		while(file.exists()) {
	            			file = new File(new File("skins"), loc + "_" + (++z) + ".png");
	            		}
	            		InputStream in = null;
	            		try {
	            			in = FileUtils.getStream(EntityNPC.getSkinTexture(entry.getLocation()));
	            			Files.copy(in, file.toPath());
	            		}
	            		catch(Exception e) {
	            			if(e instanceof FileNotFoundException)
								Log.warn("Textur ist nicht zum Kopieren vorhanden: " + FileUtils.getStream(EntityNPC.getSkinTexture(entry.getLocation())));
							else
	                    		Log.error("Konnte Textur nicht kopieren", e);
	            		}
	            		finally {
							FileUtils.closeQuietly(in);
						}
                    	this.refreshList();
                    }
            	}
            }
            else if (button.id == 30)
            {
//            	this.update = 2;
            }
            else if (button instanceof ButtonPart)
            {
                ModelPart part = ((ButtonPart)button).part;
//                this.switchModelPartEnabled(part);
                if(!this.parts.contains(part))
                    this.parts.add(part);
                else
                    this.parts.remove(part);
                if(this.gm.thePlayer != null) {
                    int n = 0;
                    for(ModelPart p : this.parts) {
                        n |= p.getMask();
                    }
                    this.gm.thePlayer.sendQueue.addToSendQueue(new CPacketAction(CPacketAction.Action.SET_MODELPARTS, n));
                }
                button.display = this.formatPart(part);
//                this.update = 2;
                if(part == ModelPart.ARMS_SLIM)
                	this.convertButton.display = "Konvertieren: " + 
                    		(this.parts.contains(ModelPart.ARMS_SLIM) ? "Schlank" : "Standard");
            }
            else if (button instanceof ButtonAlign)
            {
            	if(this.gm.thePlayer != null) {
                    this.gm.thePlayer.sendQueue.addToSendQueue(new CPacketAction(CPacketAction.Action.SET_ALIGN, 
                    		((ButtonAlign)button).align.ordinal()));
                    for(Button btn : this.buttonList) {
                    	if(btn instanceof ButtonAlign)
                    		btn.enabled = ((ButtonAlign)button).align != ((ButtonAlign)btn).align;
                    }
            	}
            }
        }
    }

    public void refreshList()
    {
        this.gm.displayGuiScreen(new GuiSkin(this.parentScreen));
    }
    
    protected void keyTyped(char typedChar, int keyCode) throws IOException
    {
		if(this.nameField.isFocused() && keyCode == Keyboard.KEY_RETURN || keyCode == Keyboard.KEY_NUMPADENTER) {
			this.sendNameToServer();
        	this.nameField.setFocused(false);
		}
		else {
			this.nameField.textboxKeyTyped(typedChar, keyCode);
		}
        if(keyCode == Keyboard.KEY_F5)
            this.refreshList();
        else
            super.keyTyped(typedChar, keyCode);
    }
    
    public void drawScreen(int mouseX, int mouseY, float partialTicks)
    {
		this.drawBackground();
//		this.drawGradientRect(0, 0, this.width, this.height - 32, 0xC0101010, 0xD0101010);
        this.listSelector.drawScreen(mouseX, mouseY, partialTicks);
//        this.drawBackground(0, this.height - 32, this.width, 32);
//        drawRect(284, 64, this.width - 284, this.height - 64, 0x20000000);
        FontRenderer.drawCenteredString("Charakter anpassen", this.width / 2, 20, 0xffffff);
    	FontRenderer.drawStringWithShadow("Anzeigename", this.nameField.xPosition, this.nameField.yPosition - 10, 10526880);
    	this.nameField.drawTextBox();
        super.drawScreen(mouseX, mouseY, partialTicks);
        GuiInventory.drawEntity(274 + (this.width - 274 - 274) / 2, this.height / 2 + 80, 80.0f /* *
        		1.8f */ / this.gm.thePlayer.getHeight(), this.yaw, this.pitch, this.gm.thePlayer);
    }
    
    protected void mouseClicked(int mouseX, int mouseY, int mouseButton) throws IOException
    {
        super.mouseClicked(mouseX, mouseY, mouseButton);
        this.listSelector.mouseClicked(mouseX, mouseY, mouseButton);
		boolean flag = this.nameField.isFocused();
        this.nameField.mouseClicked(mouseX, mouseY, mouseButton);
        if(flag && !this.nameField.isFocused())
			this.sendNameToServer();
//		else if(!flag && this.nameField.isFocused())
//        	this.nameField.setText(TextColor.getCodes(this.nameField.getText()));
    }
    
    protected void mouseReleased(int mouseX, int mouseY, int state)
    {
        super.mouseReleased(mouseX, mouseY, state);
        this.listSelector.mouseReleased(mouseX, mouseY, state);
    }

    private String formatPart(ModelPart part)
    {
        return part.getName() + ": " + (
        		this.parts.contains(part) ? (part == ModelPart.ARMS_SLIM ? "Schlank" : "An") : 
        			(part == ModelPart.ARMS_SLIM ? "Standard" : "Aus"));
    }
}
