package game.gui.world;

import java.io.File;

import game.color.TextColor;
import game.gui.Gui;
import game.gui.element.ActButton;
import game.gui.element.Label;
import game.gui.element.Textbox;
import game.gui.element.ActButton.Mode;
import game.gui.element.Textbox.Action;
import game.network.NetHandlerPlayServer;
import game.world.Region;

public class GuiEdit extends Gui implements ActButton.Callback, Textbox.Callback {
	public static interface Callback {
		void confirm(String name);
	}
	
	private final String original;
	private final boolean player;
	private final String title;
	private final String action;
	private final String desc;
	private final Callback callback;
	
	private Textbox nameField;
	private ActButton actionButton;
	private ActButton cancelButton;
	private Label actionLabel;
    private boolean fileExists;
    private boolean noChange;

	public GuiEdit(String original, boolean player, String title, String action, String desc, Callback callback) {
		this.original = original;
		this.player = player;
		this.title = title;
		this.action = action;
		this.desc = desc;
		this.callback = callback;
	}

	public void updateScreen() {
		this.actionButton.enabled = true;
		this.fileExists = false;
		this.noChange = false;
		String text = this.nameField.getText().trim();
		if(text.isEmpty() || (this.noChange = (text.equals(this.original) || (!this.player && !text.replaceAll("[\\./\"]", "_")
				.equals(text))))) {
			this.actionButton.enabled = false;
			this.actionLabel.setText(this.getLabelDesc());
			return;
		}
		if(this.player) {
			this.actionLabel.setText(this.getLabelDesc());
			return;
		}
//		{
//			if(!StringValidator.isValidUser(text))
//				this.actionButton.enabled = false;
//		}
//		else {
//		for(String n : GuiCreate.DISALLOWED_FILES) {
//			if(text.equalsIgnoreCase(n)) {
//				this.actionButton.enabled = false;
//				return;
//			}
//		}
		if(this.fileExists = (!text.isEmpty() && new File(Region.SAVE_DIR, text).exists()))
			this.actionButton.enabled = false;
		this.actionLabel.setText(this.getLabelDesc());
//		}
	}

	public void init(int width, int height) {
		this.actionButton = this.add(new ActButton(width / 2 - 100, height / 4 + 96 + 12, 200, 20, (ActButton.Callback)this, this.action));
		this.cancelButton = this.add(new ActButton(width / 2 - 100, height / 4 + 120 + 12, 200, 20, (ActButton.Callback)this, "Abbrechen"));
		this.nameField = this.add(new Textbox(width / 2 - 100, 60, 200, 20, this.player ? NetHandlerPlayServer.MAX_USER_LENGTH : 256, true, this, this.player ? NetHandlerPlayServer.VALID_USER : GuiWorlds.VALID_FILE, this.original == null ? "" : this.original));
		this.nameField.setSelected();
//		if(this.player) {
//			this.nameField.setMaxStringLength(16);
//			this.nameField.setValidator(new Predicate<String>() {
//				public boolean apply(String name) {
//					return NetHandlerPlayServer.isValidUser(name);
//				}
//			});
//		}
//		else {
//			this.nameField.setValidator(new Predicate<String>() {
//				public boolean apply(String name) {
//					for(int z = 0; z < name.length(); z++) {
//						if(GuiCreate.DISALLOWED_CHARS.contains(name.charAt(z)))
//							return false;
//					}
//					return true;
//				}
//			});
//		}
		this.actionLabel = this.add(new Label(width / 2 - 100, 20, 200, 24, this.getLabelDesc()));
//		this.shift();
//		this.nameField.setText();
		this.actionButton.enabled = false;
	}
	
	public String getTitle() {
		return "Welt bearbeiten";
	}

	public void use(ActButton btn, Mode mode) {
		this.callback.confirm(btn == this.actionButton ? this.nameField.getText().trim() : null);
	}

	public void use(Textbox elem, Action action) {
		if(action == Action.SEND)
			this.use(this.actionButton, Mode.PRIMARY);
	}
	
//	protected void actionPerformed(Button button) throws IOException {
//		if(button.enabled) {
//			if(button.id == 1) {
//			}
//			else if(button.id == 0) {
//				this.callback.confirm(this.nameField.getText().trim());
////				this.gm.displayGuiScreen(this.parent);
//			}
//		}
//	}

//	protected void keyTyped(char typedChar, int keyCode) throws IOException {
//		((Button)this.buttonList.get(0)).enabled = this.nameField.getText().trim().length() > 0;
////
////		if(keyCode == 28 || keyCode == 156) {
////			this.actionPerformed((Button)this.buttonList.get(0));
////		}
//	}

	private String getLabelDesc() {
		return (this.actionButton.enabled ? "" : TextColor.DRED) + this.desc + 
				(this.fileExists ? " - Existiert bereits" : (this.noChange ? " - Nicht geändert" : ""));
	}
}
