package game.renderer.tileentity;

import game.block.Block;
import game.gui.Font;
import game.init.Blocks;
import game.renderer.Drawing;
import game.renderer.GlState;
import game.renderer.Drawing.Vec2i;
import game.renderer.model.ModelSign;
import game.tileentity.TileEntitySign;
import game.window.WCF;


public class TileEntitySignRenderer extends TileEntitySpecialRenderer<TileEntitySign>
{
    private static final String SIGN_TEXTURE = "textures/blocks/sign.png";

    /** The ModelSign instance for use in this renderer */
    private final ModelSign model = new ModelSign();

    public void renderTileEntityAt(TileEntitySign te, double x, double y, double z, float partialTicks, int destroyStage)
    {
        Block block = te.getBlockType();
        WCF.glPushMatrix();
        float f = 0.6666667F;

        if (block == Blocks.sign)
        {
            WCF.glTranslatef((float)x + 0.5F, (float)y + 0.75F * f, (float)z + 0.5F);
            float f1 = (float)(te.getBlockMetadata() * 360) / 16.0F;
            WCF.glRotatef(-f1, 0.0F, 1.0F, 0.0F);
            this.model.signStick.showModel = true;
        }
        else
        {
            int k = te.getBlockMetadata();
            float f2 = 0.0F;

            if (k == 2)
            {
                f2 = 180.0F;
            }

            if (k == 4)
            {
                f2 = 90.0F;
            }

            if (k == 5)
            {
                f2 = -90.0F;
            }

            WCF.glTranslatef((float)x + 0.5F, (float)y + 0.75F * f, (float)z + 0.5F);
            WCF.glRotatef(-f2, 0.0F, 1.0F, 0.0F);
            WCF.glTranslatef(0.0F, -0.3125F, -0.4375F);
            this.model.signStick.showModel = false;
        }

        if (destroyStage >= 0)
        {
            this.bindTexture(DESTROY_STAGES[destroyStage]);
            WCF.glMatrixMode(5890);
            WCF.glPushMatrix();
            WCF.glScalef(4.0F, 2.0F, 1.0F);
            WCF.glTranslatef(0.0625F, 0.0625F, 0.0625F);
            WCF.glMatrixMode(5888);
        }
        else
        {
            this.bindTexture(SIGN_TEXTURE);
        }

        GlState.enableRescaleNormal();
        WCF.glPushMatrix();
        WCF.glScalef(f, -f, -f);
        this.model.renderSign();
        WCF.glPopMatrix();
//        FontRenderer fontrenderer = this.getFontRenderer();
        float f3 = 0.015625F * f;
        WCF.glTranslatef(0.0F, 0.5F * f, 0.07F * f);
        WCF.glScalef(f3, -f3, f3);
        WCF.glNormal3f(0.0F, 0.0F, -1.0F * f3);
        GlState.depthMask(false);
        int i = 0;

        if (destroyStage < 0)
        {
            for (int j = 0; j < te.signText.length; ++j)
            {
                if (te.signText[j] != null)
                {
//                    Text ichatcomponent = new Text(te.signText[j]);
//                    List<Text> list = ichatcomponent.split(90, false);
                    String s = te.signText[j].length() > 50 ? te.signText[j].substring(0, 50) : te.signText[j]; // list != null && list.size() > 0 ? ((Text)list.get(0)).getFormattedText() : "";

//                    if (j == te.lineBeingEdited)
//                    {
//                        s = "> " + s + " <";
//                        SKC.drawString(s, -SKC.getStringWidth(s) / 2, j * 10 - te.signText.length * 5, i);
//                    }
//                    else
//                    {
                    WCF.glPushMatrix();
                    WCF.glScalef(0.75f, 0.75f, 0.75f);
                    this.drawdString(s, -this.getdStringWidth(s) / 2, j * (Font.YGLYPH - 3) - 32);
                    WCF.glPopMatrix();
//                    }
                }
            }
        }

        GlState.depthMask(true);
        GlState.color(1.0F, 1.0F, 1.0F, 1.0F);
        WCF.glPopMatrix();

        if (destroyStage >= 0)
        {
            WCF.glMatrixMode(5890);
            WCF.glPopMatrix();
            WCF.glMatrixMode(5888);
        }
    }
	
	
	private void drawdString(String text, int x, int y) {
		Drawing.txt_draw(x, y, x, y, x + 400, y + 200, 0xff000000, text);
	} // TODO: signs
	private int getdStringWidth(String text) {
		Vec2i size = Drawing.txt_size(0, 0, 0, 0, 65536, 65536, text);
		return size.xpos;
	}
}
